'''
Created on 28 May 2020

@author: jacklok
'''

import math
from flask import current_app
from six import string_types
import urllib.parse

class Pager:
    def __init__(self, page, count, limit, show_only_next_and_previous=False, **kwargs):
        self.page   = page
        self.count  = count
        self.limit  = limit
        self.offset = (page-1) * limit
        self.visible_page_count = current_app.config['VISIBLE_PAGE_COUNT']
        self.page_size = limit
        self.pages = list()
        self.last_page = int(math.ceil(self.count / self.page_size))
        self.show_only_next_and_previous = show_only_next_and_previous
        self.params = kwargs
        

    def get_pages(self):
        prev_navigation, next_navigation = self.get_navigation()
        if self.show_only_next_and_previous==False:
            self.prepare_visible_pages()
            self.pages = prev_navigation + self.pages + next_navigation
        else:
            self.pages = prev_navigation + next_navigation
            
        return self.pages

    def get_navigation(self):
        prev_naviation = [{'page_label': '&laquo;',
                           'href': self.page - 1,
                           'class': 'disabled' if self.page == 1 else ''}]
        next_navigation = [{'page_label': '&raquo;',
                            'href': self.page + 1,
                            'class': 'disabled' if self.page >= self.last_page else ''}]
        return prev_naviation, next_navigation

    def prepare_visible_pages(self):
        if self.last_page > self.visible_page_count:
            self.prepare_first_pages()
            self.append_current_page()
            pages_left = self.visible_page_count - len(self.pages)
            self.prepare_last_pages(pages_left)
        else:
            self.prepare_full_range()

    def prepare_first_pages(self):
        middle_page = min(self.visible_page_count, self.last_page) // 2
        if self.page <= middle_page + 1:
            self.extend_by_range(1, self.page)
        else:
            first_visible_page = self.page - (middle_page - 2)
            self.prepend_first_page(first_visible_page)
            self.extend_by_range(first_visible_page, self.page)

    def append_current_page(self):
        self.pages.append({'page_label': self.page,
                           'href': self.page,
                           'class': 'active'})

    def prepare_last_pages(self, pages_left):
        if self.last_page - self.page <= pages_left:
            self.extend_by_range(self.page + 1, self.last_page + 1)

            if self.pages[1]['page_label'] == '...':
                while len(self.pages) < self.visible_page_count:
                    first_visible_page = self.pages[2]['page_label'] - 1
                    self.pages.insert(2, {'page_label': first_visible_page,
                                          'href': first_visible_page,
                                          'class': ''})
                    self.pages[1]['href'] = first_visible_page - 1
        else:
            last_visible_page = self.page + (pages_left - 2)
            self.extend_by_range(self.page + 1, last_visible_page + 1)
            self.append_last_page(last_visible_page)

    def extend_by_range(self, low_bound, high_bound):
        for p in range(low_bound, high_bound):
            self.pages.append({'page_label': p,
                               'href': p,
                               'class': ''})

    def prepend_first_page(self, first_visible_page):
        self.pages.append({'page_label': 1,
                           'href': 1,
                           'class': ''})
        self.pages.append({'page_label': '...',
                           'href': first_visible_page - 1,
                           'class': ''})

    def append_last_page(self, last_visible_page):
        self.pages.append({'page_label': '...',
                           'href': last_visible_page + 1,
                           'class': ''})
        self.pages.append({'page_label': self.last_page,
                           'href': self.last_page,
                           'class': ''})

    def prepare_full_range(self):
        self.extend_by_range(1, self.last_page + 1)
        self.pages[self.page - 1]['class'] = 'active'
        
    def get_params(self):
        params_str = ''
        if self.params:
            
            for k,v in self.params.items():
                params_str+='&%s=%s' % (k,urllib.parse.quote_plus(v))
                
                return '?'+params_str
        return params_str    
        
        
class CursorPager:
    def __init__(self, page, count, limit, next_cursor=None, previous_cursor=None, **kwargs):
        if isinstance(page, string_types):
            page = int(page, 10)
            
        if isinstance(count, string_types):
            count = int(count, 10)
            
        if isinstance(limit, string_types):
            limit = int(limit, 10)        
        
        self.page   = page
        self.count  = count
        self.limit  = limit
        self.offset = (page-1) * limit
        self.pages = list()
        self.page_size = limit
        self.last_page = int(math.ceil(self.count / self.page_size))
        self.next_cursor        = next_cursor
        self.previous_cursor    = previous_cursor
        self.params             = kwargs
        
    def get_pages(self):
        prev_navigation, next_navigation = self.get_navigation()
        return prev_navigation + next_navigation

    def get_navigation(self):
        prev_naviation = [{'page_label': '&laquo;',
                           'href': self.page - 1, 'cursor': self.previous_cursor if self.previous_cursor and self.page > 2 else 'None',
                           'class': 'disabled' if self.page == 1 else ''}]
        next_navigation = [{'page_label': '&raquo;',
                            'href': self.page + 1, 'cursor': self.next_cursor if self.next_cursor else 'None',
                            'class': 'disabled' if self.page >= self.last_page else ''}]
        return prev_naviation, next_navigation
    
    def get_params(self):
        params_str = ''
        if self.params:
            
            for k,v in self.params.items():
                params_str+='&%s=%s' % (k,urllib.parse.quote_plus(v))
                
                return '?'+params_str
        return params_str
